// src/index.ts
import {normalizePath} from "vite";

// src/createMockServer.ts
import {existsSync} from "fs";
import {join} from "path";
import chokidar from "chokidar";
import chalk from "chalk";
import url from "url";
import fg from "fast-glob";
import Mock from "mockjs";

// src/utils.ts
import path from "path";
var toString = Object.prototype.toString;
function is(val, type) {
  return toString.call(val) === `[object ${type}]`;
}
function isFunction(val) {
  return is(val, "Function");
}
function isArray(val) {
  return val && Array.isArray(val);
}
function isRegExp(val) {
  return is(val, "RegExp");
}
function isBoolean(val) {
  return is(val, "Boolean");
}
function sleep(time) {
  return new Promise((resolve) => {
    setTimeout(() => {
      resolve("");
    }, time);
  });
}
async function loadConfigFromBundledFile(fileName, bundledCode) {
  const extension = path.extname(fileName);
  const defaultLoader = require.extensions[extension];
  require.extensions[extension] = (module, filename) => {
    if (filename === fileName) {
      module._compile(bundledCode, filename);
    } else {
      defaultLoader(module, filename);
    }
  };
  delete require.cache[fileName];
  const raw = require(fileName);
  const config = raw.__esModule ? raw.default : raw;
  require.extensions[extension] = defaultLoader;
  return config;
}

// src/createMockServer.ts
import {rollup} from "rollup";
import esbuildPlugin from "rollup-plugin-esbuild";
import dayjs from "dayjs";
var pathResolve = require("@rollup/plugin-node-resolve");
var mockData = [];
async function createMockServer(opt = {mockPath: "mock", ignoreFiles: [], configPath: "vite.mock.config"}) {
  opt = {
    mockPath: "mock",
    ignoreFiles: [],
    watchFiles: true,
    supportTs: true,
    configPath: "vite.mock.config.ts",
    ...opt
  };
  if (mockData.length > 0)
    return;
  mockData = await getMockConfig(opt);
  const {watchFiles = true} = opt;
  if (watchFiles) {
    const watch = await createWatch(opt);
    watch && watch();
  }
}
function getInvokeTime(opt) {
  const {showTime} = opt;
  const defTime = ` - ${dayjs().format("YYYY-MM-DD HH:mm:ss")}`;
  if (isBoolean(showTime)) {
    return !showTime ? "" : defTime;
  }
  if (!showTime)
    return "";
  try {
    return dayjs().format(showTime);
  } catch (error) {
    return defTime;
  }
}
function requestMiddle(opt) {
  const middleware = async (req, res, next) => {
    let queryParams = {};
    const isGet = req.method === "GET";
    if (isGet && req.url) {
      queryParams = url.parse(req.url, true);
    }
    const matchReq = mockData.find((item) => {
      if (isGet) {
        return item.url === queryParams.pathname;
      }
      return item.url === req.url;
    });
    if (matchReq) {
      const {response, timeout, statusCode} = matchReq;
      if (timeout) {
        await sleep(timeout);
      }
      const {query = {}} = queryParams;
      const body = req.body;
      const mockRes = isFunction(response) ? response({body, query}) : response;
      console.log(`${chalk.green(`[vite:mock-server${getInvokeTime(opt)}]:request invoke:  ${chalk.cyan(req.url)} `)}`);
      res.setHeader("Content-Type", "application/json");
      res.statusCode = statusCode || 200;
      res.end(JSON.stringify(Mock.mock(mockRes)));
      return;
    }
    next();
  };
  return middleware;
}
function createWatch(opt) {
  const {configPath} = opt;
  const {absConfigPath, absMockPath} = getPath(opt);
  if (process.env.VITE_DISABLED_WATCH_MOCK === "true")
    return;
  const watchDir = [];
  const exitsConfigPath = existsSync(absConfigPath);
  exitsConfigPath && configPath ? watchDir.push(absConfigPath) : watchDir.push(absMockPath);
  const watcher = chokidar.watch(watchDir, {
    ignoreInitial: true
  });
  const watch = () => {
    watcher.on("all", async (event, file) => {
      console.log(chalk.magentaBright(`[vite:mock-server]:mock file ${event}.`));
      mockData = await getMockConfig(opt);
    });
  };
  return watch;
}
function cleanRequireCache(opt) {
  const {absConfigPath, absMockPath} = getPath(opt);
  Object.keys(require.cache).forEach((file) => {
    if (file === absConfigPath || file.indexOf(absMockPath) > -1) {
      delete require.cache[file];
    }
  });
}
async function getMockConfig(opt) {
  cleanRequireCache(opt);
  const {absConfigPath, absMockPath} = getPath(opt);
  const {ignoreFiles = [], ignore, configPath, supportTs} = opt;
  let ret = [];
  if (configPath && existsSync(absConfigPath)) {
    console.log(chalk.blue(`[vite:mock-server]:load mock data from ${absConfigPath}`));
    let resultModule = await resolveModule(absConfigPath);
    ret = resultModule;
  } else {
    const mockFiles = fg.sync(`**/*.${supportTs ? "ts" : "js"}`, {
      cwd: absMockPath,
      ignore: ignoreFiles
    }).filter((item) => {
      if (!ignore) {
        return true;
      }
      if (isFunction(ignore)) {
        return ignore(item);
      }
      if (isRegExp(ignore)) {
        return !ignore.test(item);
      }
      return true;
    });
    try {
      ret = [];
      for (let index = 0; index < mockFiles.length; index++) {
        const mockFile = mockFiles[index];
        const resultModule = await resolveModule(join(absMockPath, mockFile));
        let mod = resultModule;
        if (!isArray(mod)) {
          mod = [mod];
        }
        ret = [...ret, ...mod];
      }
    } catch (error) {
      console.log(`${chalk.red("[vite:mock-server]:mock reload error!")}
${error}`);
      ret = [];
    }
  }
  return ret;
}
async function resolveModule(path3) {
  const nodeResolve = pathResolve.nodeResolve({
    extensions: [".js", ".ts"]
  });
  const bundle = await rollup({
    input: path3,
    treeshake: false,
    plugins: [
      esbuildPlugin({
        include: /\.[jt]sx?$/,
        exclude: /node_modules/,
        sourceMap: false
      }),
      nodeResolve
    ]
  });
  const {
    output: [{code}]
  } = await bundle.generate({
    exports: "named",
    format: "cjs"
  });
  return await loadConfigFromBundledFile(path3, code);
}
function getPath(opt) {
  const {mockPath, configPath} = opt;
  const cwd = process.cwd();
  const absMockPath = join(cwd, mockPath || "");
  const absConfigPath = join(cwd, configPath || "");
  return {
    absMockPath,
    absConfigPath
  };
}

// src/index.ts
import bodyParser from "body-parser";
import path2 from "path";
import babel from "@babel/core";
function viteMockServe(opt) {
  const {supportTs = true} = opt;
  const {
    injectFile = normalizePath(path2.resolve(process.cwd(), `src/main.${supportTs ? "ts" : "js"}`))
  } = opt;
  let isDev = false;
  let config;
  let needSourceMap = false;
  return {
    name: "vite:mock",
    enforce: "pre",
    configResolved(resolvedConfig) {
      config = resolvedConfig;
      isDev = config.command === "serve" && !config.isProduction;
      needSourceMap = config.isProduction && !!config.build.sourcemap;
    },
    configureServer: ({middlewares}) => {
      const {localEnabled = isDev} = opt;
      if (!localEnabled)
        return;
      createMockServer(opt);
      middlewares.use(bodyParser.urlencoded({extended: false}));
      middlewares.use(bodyParser.json());
      middlewares.use(requestMiddle(opt));
    },
    async transform(code, id) {
      let map = null;
      if (needSourceMap) {
        const plugins = [];
        if (id.endsWith(".tsx")) {
          plugins.push([
            require("@babel/plugin-transform-typescript"),
            {isTSX: true, allowExtensions: true}
          ]);
        }
        const result = babel.transformSync(code, {
          ast: false,
          plugins,
          sourceMaps: true,
          sourceFileName: id
        });
        map = result?.map;
      }
      const defResult = {
        code,
        map
      };
      if (isDev || !id.endsWith(injectFile))
        return defResult;
      const {prodEnabled = true, injectCode = ""} = opt;
      if (!prodEnabled)
        return defResult;
      return {
        code: `
      ${code}
      

      ${injectCode}
      `,
        map
      };
    }
  };
}
export {
  viteMockServe
};
